from .bases import BaseJsonPage, BaseJsonBlock
from .mixins import AIslandGetThreadId
import re

__all__ = ['AdnmbBlock', 'AdnmbPage']


_request_info = {
    'cdn_host': 'http://h-adnmb-com.n1.yun.tf:8999/Public/Upload',
    'headers': {
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
        'Accept-Encoding': 'gzip, deflate, sdch',
        'Accept-Language': 'en-US,en;q=0.8,zh-CN;q=0.6,zh;q=0.4',
        'Cache-Control': 'no-cache',
        'Connection': 'keep-alive',
        'Dnt': '1',
        'Host': 'h-adnmb-com.n1.yun.tf:8999',
        'Pragma': 'no-cache',
        'Referer': 'http://h.adnmb.com/',
        'Upgrade-Insecure-Requests': '1',
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/52.0.2743.116 Safari/537.36'
    }
}


class AdnmbBlock(BaseJsonBlock):
    request_info = _request_info

    def _deal_with_reply(self, content):
        return re.sub(r'(<font color.*?>.*?\d+</font>)', r'<span class="reply-color">\1</span>', content)

class AdnmbPage(AIslandGetThreadId, BaseJsonPage):
    block_model = AdnmbBlock


from .basetest import *

class TestKukuku(BaseTest):
    RAW_URLS = ['http://www.kukuku.cc/t/6934861/1', 'http://www.kukuku.cc/t/6934861']
    REQUEST_URLS = ['http://www.kukuku.cc/t/6934861/1', 'http://www.kukuku.cc/t/6934861/1']
    NEXT_PAGE_URL = 'http://www.kukuku.cc/t/6934861/2'
    THREAD_LIST_NUM = 21
    BLOCK_0_DATA = {
        'image_url': 'http://static.kukuku.cc/FoU6oNXFNKbMNvzAw_kRfmcFeze5.jpeg',
        'uid': 'JxpxDCEjXM',
        'id': '6934861',
        'created_time': '2016-09-07 17:42:39',
        'content': '''(;´Д`)宿舍门口有只超骚的小主子，想要撩撩它，应该拿什么喂它或者逗它(=ﾟωﾟ)=''',
    }
    BLOCK_1_DATA = {
        'image_url': None,
        'uid': 'WudUsohuEP',
        'id': '10209793',
        'created_time': '2016-09-07 18:30:29',
        'content': """斯大林主子(ﾟДﾟ≡ﾟДﾟ)""",
    }

    def test_another_block(self):
        pass


from island_backup import network
from island_backup.island_switcher import island_switcher
import aiohttp
import asyncio
import pytest


async def get_page(url):
    network.session = aiohttp.ClientSession()
    island_switcher.detect_by_url(url)
    url = island_switcher.sanitize_url(url)
    print(url)
    p = await island_switcher.island_page_model.from_url(url, page_num=1)
    network.session.close()
    return p


def check_block(block, data_dict):
    check_key = ['uid', 'id', 'content', 'image_url', 'created_time']
    for key in check_key:
        assert getattr(block, key) == data_dict[key]


class BaseTest:
    NEXT_PAGE_URL = ''
    THREAD_LIST_NUM = 20
    BLOCK_0_DATA = None
    BLOCK_1_DATA = None
    # 原始用户输入的URL
    RAW_URLS = None
    # 转换后程序应该请求的URL， 与RAW_URLS一一对应。
    REQUEST_URLS = None

    @pytest.fixture(scope='class')
    def page(self):
        return asyncio.get_event_loop().run_until_complete(get_page(self.RAW_URLS[0]))

    @pytest.fixture(scope='class')
    def thread_list(self, page):
        return page.thread_list()


    def test_sanitize_url(self, page):
        for raw, req in zip(self.RAW_URLS, self.REQUEST_URLS):
            assert page.url_page_combine(page.sanitize_url(raw), 1) == req

    def test_page(self, page):
        assert page.has_next()
        print(page.next_page_info)
        assert page.url_page_combine(*page.next_page_info) == self.NEXT_PAGE_URL

    def test_blocks_num(self, thread_list):
        print(len(thread_list))
        assert len(thread_list) == self.THREAD_LIST_NUM


    def test_first_block(self, thread_list):
        block = thread_list[0]
        check_block(block, self.BLOCK_0_DATA)

    def test_second_block(self, thread_list):
        block = thread_list[1]
        check_block(block, self.BLOCK_1_DATA)

    def test_another_block(self, thread_list):
        raise NotImplementedError



from .bases import BasePage, BaseBlock
from bs4 import BeautifulSoup
import urllib.parse
import re
from .mixins import AIslandGetThreadId


class KukukuBlock(BaseBlock):
    request_info = {
        'cdn_host': 'http://static.kukuku.cc/',
        'headers': {
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
            'Accept-Encoding': 'gzip, deflate, sdch',
            'Accept-Language': 'en-US,en;q=0.8,zh-CN;q=0.6,zh;q=0.4',
            'Cache-Control': 'no-cache',
            'Connection': 'keep-alive',
            'Dnt': '1',
            'Host': 'static.kukuku.cc',
            'Pragma': 'no-cache',
            'Referer': 'http://www.kukuku.cc/t/6645621',
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/49.0.2623.110 Safari/537.36'
            }
    }

    @property
    def id(self):
        return self._block.find('a', class_='h-threads-info-id').text.split('.')[-1]

    @property
    def uid(self):
        return self._block.find('span', class_='h-threads-info-uid').text.split(':')[-1]

    def _get_content(self):
        div = self._block.find('div', class_='h-threads-content')
        return ''.join(str(e).strip() for e in div.contents)

    def _deal_with_reply(self, content):
        return re.sub(r'(<span class=.*?>.*?\d+</span>)', r'<span class="reply-color">\1</span>', content)

    @property
    def created_time(self):
        return self._block.find('span', class_='h-threads-info-createdat').text

    @property
    def image_url(self):
        tag = self._block.find('a', class_='h-threads-img-a')
        if tag:
            return tag.attrs.get('href')
        else:
            return None


class KukukuPage(AIslandGetThreadId, BasePage):
    block_model = KukukuBlock

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.bs = BeautifulSoup(self.data, 'html.parser')
        # self.bs = BeautifulSoup(self.data, 'lxml')

    @staticmethod
    def url_page_combine(base_url, page_num):
        return urllib.parse.urljoin(base_url, str(page_num))

    def thread_list(self):
        top = self.block_model(self.bs.find(class_='h-threads-item-main'))
        threads = [self.block_model(b) for b in self.bs.find_all(class_='h-threads-item-reply')]
        threads.insert(0, top)
        return threads

    def has_next(self):
        if self.bs.find('a', text='下一页'):
            return True
        return False

    @property
    def total_page(self):
        return None

    @staticmethod
    def sanitize_url(url):

        # find the base url, add '/' at the end to mark this url a base_url
        url = re.match('.*/t/\d+', url).group(0)
        url = url + '/'
        return url




# import pytest
# pytest.mark
#
# k = 0
#
# class BaseTest:
#     c=None
#     @pytest.fixture(scope='class')
#     def page(self):
#         print('get page')
#         global k
#         k+=1
#         return self.c
#
#     @pytest.fixture(scope='class')
#     def g(self, page):
#         return page + 1
#
#     def test_c(self, page, g):
#         assert 1 == page
#         assert 2 == g
#
#     def test_d(self, page):
#         print(k)
#
#
# class TestA(BaseTest):
#     c=1
#
#
# class TestB(BaseTest):
#     c=2
#
#     def test_c(self, page, g):
#         assert 2==page
#         assert g==3
#
#     def test_r(self, g):
#         print(g)
#
#     def test_b(self, page):
#         print(k)
#         assert 0
#
#
#
# a = 1+5
#
# def test_a():
#     print(1)
#
#     assert a==3



import json
import os.path



SEARCH_PATH = ['.', os.path.expanduser('~/.island_backup')]
CONFIG_FILENAME = 'island_backup.json'


settings = {
    'proxy': None,
    'debug': False,
    'force-update': False,
    'conn-count': 20,
}


def load_config():
    for basepath in SEARCH_PATH:
        filename = os.path.join(basepath, CONFIG_FILENAME)
        if os.path.exists(filename):
            try:
                with open(filename, 'r', encoding='utf8') as f:
                    user_settings = json.load(f)
            except json.JSONDecodeError:
                print("ERROR: Can not decode {}!\n".format(filename))
            except:
                print('WARNING: Unexcept Error in {}, Ignore.'.format(filename))

            else:
                print('Find island_backup.json at {}'.format(filename))
                return user_settings
    return None


user_settings = load_config()
if user_settings:
    settings.update(user_settings)





from tests.basetest import *


class TestNiMingBan(BaseTest):

    REQUEST_URLS = ['https://h.nimingban.com/t/9800114?page=1']
    RAW_URLS = ['https://h.nimingban.com/t/9800114']
    NEXT_PAGE_URL = 'https://h.nimingban.com/t/9800114?page=2'
    THREAD_LIST_NUM = 20
    BLOCK_0_DATA = {
        'image_url': 'http://cdn.ovear.info:8998/image/2016-09-18/57de3d443bce3.jpg',
        'uid': '4ZzUhjy',
        'id': '9800114',
        'created_time': '2016-09-18 15:07:48',
        'content': """下个星期就考了，好慌"""
    }
    BLOCK_1_DATA = {
        'image_url': None,
        'uid': 'JvnJBG5',
        'id': '9800148',
        'created_time': '2016-09-18 15:11:07',
        'content': '选b啊！！'
    }

    def test_another_block(self, thread_list):
        b = thread_list[4]
        assert b.content == """<span class="reply-color"><font color="#789922">&gt;&gt;No.9800114</font></span><br/>……出题老师可以的"""



from .basetest import *


class Test4Chan(BaseTest):
    RAW_URLS = ['http://boards.4chan.org/u/thread/1436960/welcome-to-u', 'http://boards.4chan.org/u/thread/1436960']
    REQUEST_URLS = ['http://a.4cdn.org/u/thread/1436960.json', 'http://a.4cdn.org/u/thread/1436960.json']
    NEXT_PAGE_URL = None
    THREAD_LIST_NUM = 5
    BLOCK_0_DATA = {
        'image_url': 'http://i.4cdn.org/u/1376789114258.jpg',
        'uid': 'Anonymous',
        'id': 1436960,
        'created_time': '08/17/13(Sat)21:25',
        'content':
            '/u/ is for discussions about the yuri genre of manga, anime, and other related media. Threads requesting images or series recommendations are discouraged (try <a href="https://boards.4chan.org/r/">>>>/r/</a> instead). When starting an image dump thread, please contribute at least 4-5 relevant images yourself. <br><br>Where to read manga: <br><a href="http://dynasty-scans.com/">http://dynasty-scans.com/</a><br><br>Some notable scanlators:<br><a href="http://www.lililicious.net/">http://www.lililicious.net/</a><br><a href="http://yuri-ism.com/">http://yuri-ism.com/</a><br><a href="http://yuriproject.net/">http://yuriproject.net/</a><br><br>Just getting into the yuri genre, or just looking for recommendations? Click [Reply] for /u/ guides to manga, anime, and live action.',

    }
    BLOCK_1_DATA = {
        'image_url': 'http://i.4cdn.org/u/1376789154203.png',
        'uid': 'Anonymous',
        'id': 1436961,
        'created_time': '08/17/13(Sat)21:25',
        'content': """These charts are intended to provide major introductory material and are by no means all-encompassing.<br><br>Where to start with yuri manga.""",
    }

    def test_another_block(self, thread_list):
        pass

    def test_page(self, page):
        pass


import re
from urllib import parse
from .bases import BaseJsonBlock, BaseJsonPage
import os.path


JSON_CDN = 'http://a.4cdn.org'

class The4ChanBlock(BaseJsonBlock):
    request_info = {
        'cdn_host': 'http://i.4cdn.org',
        'headers': {
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
            'Accept-Encoding': 'gzip, deflate, sdch',
            'Accept-Language': 'en-US,en;q=0.8,zh-CN;q=0.6,zh;q=0.4',
            'Cache-Control': 'no-cache',
            'Connection': 'keep-alive',
            'Dnt': '1',
            'Pragma': 'no-cache',
            'Host': 'i.4cdn.org',
            'Referer': 'http://boards.4chan.org/',
            'Upgrade-Insecure-Requests': '1',
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.80 Safari/537.36'
        }
    }

    def __init__(self, block_data, broad=None):
        super().__init__(block_data=block_data)
        self.broad = broad

    @property
    def id(self):
        return self._block.get('no', '')

    @property
    def uid(self):
        return self._block.get('name', '')

    def _get_content(self):
        return self._block.get('com', '')

    def _deal_with_reply(self, content):
        return content

    @property
    def image_url(self):
        tim = self._block.get('tim', None)
        if not tim:
            return None
        ext = self._block['ext']

        image_name = str(tim) + ext
        url = '/'.join((self.request_info['cdn_host'], self.broad, image_name))
        return url

    @property
    def created_time(self):
        return self._block['now']



class The4ChanPage(BaseJsonPage):
    block_model = The4ChanBlock
    def has_next(self):
        return False

    @property
    def total_page(self):
        return None

    @staticmethod
    def url_page_combine(base_url, page_num):
        return base_url

    @staticmethod
    def get_thread_id(url):
        return re.match(r'.*/thread/(\d+)', url).group(1)

    @staticmethod
    def _get_broad_short_name(url):
        return parse.urlparse(url).path.split('/')[1]

    def thread_list(self):
        posts = self.data['posts']
        broad = self._get_broad_short_name(self.base_url)
        return [self.block_model(block_data=b, broad=broad) for b in posts]

    @staticmethod
    def sanitize_url(url):
        path = re.match('.*/\d+', parse.urlparse(url).path).group() + '.json'
        return parse.urljoin(JSON_CDN, path)



import pytest
import asyncio

from tests.basetest import *





class TestAdnmb(BaseTest):
    RAW_URLS = ['http://h.adnmb.com/t/106983']
    REQUEST_URLS = ['http://h.adnmb.com/api/t/106983?page=1']
    NEXT_PAGE_URL = 'http://h.adnmb.com/api/t/106983?page=2'
    THREAD_LIST_NUM = 21
    BLOCK_0_DATA = {
        'image_url': 'http://h-adnmb-com.n1.yun.tf:8999/Public/Upload/image/2016-05-14/57372d96e74c2.jpg',
        'uid': 'W0wrx3S',
        'id': '106983',
        'content': """|∀ﾟ有广东的肥肥吗，我好想买，可惜我这里暂时没有""",
        'created_time': '2016-05-14 21:52:23'
    }
    BLOCK_1_DATA = {
        'image_url': None,
        'uid': 'dPRC8y8',
        'content': """大吃人在此，不过我用的是移动| ω・´)""",
        'id': '106987',
        'created_time': '2016-05-14 21:56:00',
    }


    def test_another_block(self, thread_list):
        block = thread_list[7]
        assert block.image_url == 'http://h-adnmb-com.n1.yun.tf:8999/Public/Upload/image/2016-05-14/57373100c6017.png'


def atest_tmp(tmpdir):
    a = tmpdir.mkdir('sub')
    with open(str(a) + '/xx.txt', 'w') as f:
        f.write('?')
    c= a.join('xx.txt')
    print(c.read())
    print(a)
    assert 0




from setuptools import setup, find_packages

with open('README.rst', 'r', encoding='utf8') as f:
    readme = f.read()

with open('requirements.txt','r',encoding='utf8') as f:
    requirements = f.readlines()

version = __import__('island_backup').version


setup(
    name='island_backup',
    version=version,
    description="backup 4chan.org h.nimingban and kukuku.cc",
    long_description=readme,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3 :: Only',
        'License :: OSI Approved :: MIT License',

    ],
    url='https://github.com/littlezz/island-backup',
    author='littlezz',
    author_email='zz.at.field@gmail.com',
    license='MIT',
    packages=find_packages(exclude=['tests*',]),
    install_requires=requirements,
    tests_require=['pytest'],
    # include_package_data=True,
    package_data={
        'island_backup': ['templates/*', 'templates/static/*']
    },
    zip_safe=False,
    entry_points={
        'console_scripts': [
            'island_backup=island_backup.main:cli'
        ]
    },

)




from .islands.adnmb import AdnmbPage
from .islands.nimingban import NiMingBanPage
from .islands.kukuku import KukukuPage
from .islands.the4chan import The4ChanPage


island_class_map = {
    'adnmb': AdnmbPage,
    'nimingban': NiMingBanPage,
    'kukuku': KukukuPage,
    '4chan': The4ChanPage,
}

class IslandSwitcher:
    available_island = island_class_map.keys()

    def __init__(self, island=None):
        self.island = island

    def detect_by_url(self, url):
        for island in self.available_island:
            if island in url:
                self.island = island
                return
        raise ValueError('Unknown url: {}'.format(url))

    @property
    def island_page_model(self):
        return island_class_map[self.island]

    def sanitize_url(self, url):
        return self.island_page_model.sanitize_url(url)

    def get_folder_name(self, url):
        return self.island + '_' + str(self.island_page_model.get_thread_id(url))


island_switcher = IslandSwitcher()